package com.example.catchTheLetters.handler;

import com.example.catchTheLetters.entity.GameMessage;
import com.example.catchTheLetters.entity.LetterAction;
import com.example.catchTheLetters.entity.PlayerInput;
import com.example.catchTheLetters.entity.RoomAction;
import com.example.catchTheLetters.enums.LetterActionType;
import com.example.catchTheLetters.enums.MessageType;
import com.example.catchTheLetters.service.RoomService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Controller;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.util.Objects;

/**
 * WebSocket 处理器
 *
 * @author spyn
 */
@Controller
public class WebSocketHandler extends TextWebSocketHandler {

    private final ObjectMapper objectMapper = new ObjectMapper();

    @Resource
    private RoomService roomService;

    @Override
    public void handleTextMessage(@NotNull WebSocketSession session, TextMessage message) {
        // 处理接收到的消息
        GameMessage<?> gameMessage;
        try {
            gameMessage = objectMapper.readValue(message.getPayload(), GameMessage.class);
        } catch (JsonProcessingException e) {
            roomService.sendMessage(session, new GameMessage<>(MessageType.ERROR, "消息解析失败"));
            return;
        }

        switch (gameMessage.getType()) {
            case INPUT:
                // 处理输入消息
                roomService.handleInput(gameMessage.getRoomId(), session, (PlayerInput) gameMessage.getData());
                break;
            case ROOM:
                // 处理房间消息
                var roomData = (RoomAction) gameMessage.getData();
                switch (roomData.getType()) {
                    case CREATE:
                        roomService.createRoom(session, roomData.getToken());
                        break;
                    case JOIN:
                        roomService.addPlayer(gameMessage.getRoomId(), session, roomData.getToken());
                        break;
                    case LEAVE:
                        roomService.removePlayer(gameMessage.getRoomId(), session);
                        break;
                    case KICK:
                        roomService.kickPlayer(gameMessage.getRoomId(), session, roomData.getUserID());
                        break;
                    case START:
                        roomService.startGame(gameMessage.getRoomId(), session);
                        break;
                    case CANCEL_START:
                        roomService.cancelStartGame(gameMessage.getRoomId(), session);
                        break;
                    case END:
                        roomService.endGame(gameMessage.getRoomId());
                        break;
                    default:
                        roomService.sendMessage(session, new GameMessage<>(MessageType.ERROR, "未知的房间操作"));
                }
                break;
            case LETTER:
                // 处理字母消息
                var letterData = (LetterAction) gameMessage.getData();
                if (Objects.requireNonNull(letterData.getType()) == LetterActionType.GET)
                    // 获取字母
                    roomService.getLetter(gameMessage.getRoomId(), session, letterData.getLetterId());
                else
                    roomService.sendMessage(session, new GameMessage<>(MessageType.ERROR, "未知的字母操作"));
                break;
            case LOGIN:
                // 处理登录消息
                roomService.connect(session, (String) gameMessage.getData());
                break;
            case LOGOUT:
                // 处理登出消息
                roomService.disconnect((String) gameMessage.getData());
                break;
            default:
                roomService.sendMessage(session, new GameMessage<>(MessageType.ERROR, "未知的消息类型"));
        }
    }
}
